function [StatesDot] = dynamics(t, States, param)

    % Extract parameters from input 'param'
    n       = param{1};
    m       = param{2};
    omega   = param{3};
    a       = param{4};
    Sigma   = param{5};
    kt      = param{6};
    K       = param{7};
    kg      = param{8};
    tau     = param{9};
    f       = param{10};
    h       = param{11};

    % Indices of different state variables within the state vector.
    i(1) = 1;
    i(2) = 2 + m;    
    i(3) = 2 + m + 1;
    i(4) = 2 + m + 2 + (1 + m).^2 - 1;
    i(5) = i(4) + m + 1;
    i(6) = i(5) + 2 * m;
    i(7) = i(6) + m;
    i(8) = i(7) + n; 

    % Extract state variables from 'States' according to the indices defined above.
    y_kapa     = States(i(1)); 
    c          = States(i(1) + 1 : i(2)); 
    ni_kapa    = States(i(3)); 
    sigma      = States(i(3) + 1 : i(4)); 
    theta_kapa = States(i(4) + 1 : i(5)); 
    d          = States(i(5) + 1 : i(6));
    u_kapa     = 0; % States(i(6) + 1 : i(7)); Note: Assignment to 0 means u_kappa is not used from States.
    x          = States(i(7) + 1 : i(8)); 

    % Compute some intermediate variables used in the dynamic equations.
    y    = h(x);
    u    = -kg * theta_kapa(2:end) + u_kapa + d(1:2:end);
    phi  = [1; u - u_kapa];
    e    = y - y_kapa;
    sigma = reshape(sigma, m + 1, m + 1);
    sigmaDot = -sigma * c * c' * sigma + kt * sigma - Sigma * sigma;

    % Construct matrix Phi using the omega values.
    Omega_cell = cell(1, m);
    for j = 1:m
        Omega_cell{j} = [0 omega(j); -omega(j) 0];
    end
    Phi = diag_matrices(Omega_cell);

    % Initialize the time derivative of state vector.
    StatesDot = zeros(size(States));

    % Calculate the time derivatives of the states.
    StatesDot(i(1) + 1 : i(2))  = -K*c + phi; % Derivative of c
    StatesDot(i(3))             = -K * ni_kapa; % Derivative of ni_kappa
    StatesDot(i(3) + 1 : i(4))  = sigmaDot(:); % Derivative of sigma
    StatesDot(i(4) + 1 : i(5))  = sigma * (c * (e - ni_kapa) - Sigma * theta_kapa); % Derivative of theta_kappa
    StatesDot(i(5) + 1 : i(6))  = Phi * d; % Derivative of d
    StatesDot(i(6) + 1 : i(7))  = 0; % -1/tau * theta_kapa(2:end); % Derivative of u_kappa, it is set to 0
    StatesDot(i(7) + 1 : i(8))  = f(x, u); % Derivative of x
    StatesDot(i(1))             = phi' * theta_kapa + K * e + c'*StatesDot(i(4) + 1 : i(5)) ;  % Derivative of y_kappa
end
