% Setting some initial parameters.
n = 2;
m = 1;
alfa = 0.1;
eye_mat = eye(1 + m); % Identity matrix of size (1+m)x(1+m).

K = 10;
tau = 20;

% Defining Q and R matrices.
Q = [1 -1/2; -1/2 1];
R = [0; -2];
Q1 = Q + 0.01 * eye(2); % Perturbed Q matrix (increased).
Q2 = Q - 0.01 * eye(2); % Perturbed Q matrix (decreased).

% Defining A and B matrices.
B = [0 -1]';
A = [-0.5 1; 0 -0.5] + 0.5 * B * B' * Q ;

% Creating M matrix using the above defined matrices.
M = [A, B; B' * Q * (1/tau * eye(2) - K * A), -K * B' * Q * B];
disp(['Is the original system stable? ', num2str(all(eig(M) < 0))])

% Creating perturbed M matrices for two different cases.
M1 = [A, B; B' * Q1 * (1/tau * eye(2) - K * A), -K * B' * Q1 * B];
disp(['Is perturbed system 1 stable? ', num2str(all(eig(M1) < 0))])
M2 = [A, B; B' * Q2 * (1/tau * eye(2) - K * A), -K * B' * Q2 * B];
disp(['Is perturbed system 2 stable? ', num2str(all(eig(M2) < 0))])

N = [0 0 -2/tau]';

% Calculating different criterion for system stability.
Aprime = A - 0.5 * B * B' * Q;
k1 = B' * (inv(Aprime))' * Q * (inv(Aprime)) * B;    % supposed criterion
k2 = B' * ((inv(Aprime))'* Q + Q * (inv(Aprime))) * B;% real criterion

% Calculating different criterion for perturbed systems stability.
k11 = B' * (inv(Aprime))' * Q1 * (inv(Aprime)) * B;  % supposed criterion for perturbed system 1
k12 = B' * ((inv(Aprime))'* Q1 + Q1 * (inv(Aprime))) * B;% real criterion for perturbed system 1

k21 = B' * (inv(Aprime))' * Q2 * (inv(Aprime)) * B;  % supposed criterion for perturbed system 2
k22 = B' * ((inv(Aprime))'* Q2 + Q2 * (inv(Aprime))) * B;% real criterion for perturbed system 2

% Setting time parameters.
T = 1;
time = 2700;

% Defining the system dynamics.
f = @(x) M * x + N * 1;
f1 = @(x) M1 * x + N * 1;
f2 = @(x) M2 * x + N * 1;

% Initial conditions for the system.
x0 = zeros(3, 1);

% Solving the system of differential equations for all three cases.
[t,y] = ode45(@(t,y)f(y), [0 time], x0);
[t1,y1] = ode45(@(t,y)f1(y), [0 time], x0);
[t2,y2] = ode45(@(t,y)f2(y), [0 time], x0);

% Plotting the results.
figure
figure_standard
hold on
xlim([0 time])
ylim([0 1200])
plot(t1, y1(:, 1), 'r', 'HandleVisibility','off', 'linewidth', 2)
plot(t1, y1(:, 2), 'r', 'linewidth', 2)
plot(t, y(:, 1), 'b', 'HandleVisibility','off', 'linewidth', 2)
plot(t, y(:, 2), 'b', 'linewidth', 2)
plot(t2, y2(:, 1), 'g', 'HandleVisibility','off', 'linewidth', 2)
plot(t2, y2(:, 2), 'g', 'linewidth', 2)
legend('Case 1', 'Case 2', 'Case 3');
xlabel('$t$')
ylabel('$x_1, x_2$')

% Calculating the solution to the perturbed system 2 and plotting it.
xsol =  -M2^(-1) * N;
plot([0, t2(end)], [xsol(1, 1), xsol(1, 1)], 'k--', 'HandleVisibility','off')
plot([0, t2(end)], [xsol(2, 1), xsol(2, 1)], 'k--', 'HandleVisibility','off')

% Defining Aprime and G for later use.
Aprime = A - K * B * B' * Q;
G = (inv(Aprime))'* Q;
