% This function takes in current time t, the state vector States_all, and a parameter cell array param.
% It returns the time derivative of the state vector.
function [StatesDot] = dynamics_INESC(t, States_all, param)
    
    % Unpack parameters from the cell array param
    n               = param{1};                  % Number of elements in system
    m               = param{2};                  % 
    omega           = param{3};                  % Frequency
    a               = param{4};
    Sigma           = param{5};
    kt              = param{6};
    K               = param{7};                  % Gain
    kg              = param{8};
    tau             = param{9};                  % Time constant
    f               = param{10};                 % Function handle to system dynamics
    h               = param{11};                 % Function handle to output function
    sistem_state_size = param{12}; 
    N               = param{13};                 % Number of systems
    
    % Define the index ranges for each system
    sistem_index    = [0, cumsum(sistem_state_size(2, :))];
    sistem_state_index = sistem_index(1:end - 1) + sistem_state_size(1, :) + 1;
    x_all           = [];
    for j = 1:N
       x_all        = [x_all; States_all(sistem_state_index(j) : sistem_index(j + 1))]; 
    end
    
    % Initialize output cell array
    StatesDot       = cell(1, N);
    
    for j = 1:N
        % Extract the state vector for the jth system
        States       = States_all(sistem_index(j) + 1 : sistem_index(j + 1));

        % Define indices for different parts of the state vector
        i(1)         = 1;
        i(2)         = 2 + m(j);    
        i(3)         = 2 + m(j) + 1;
        i(4)         = 2 + m(j) + 2 + (1 + m(j)).^2 - 1;
        i(5)         = i(4) + m(j) + 1;
        i(6)         = i(5) + 2 * m(j);
        i(7)         = i(6) + m(j);
        i(8)         = i(7) + n(j); 

        % Unpack state variables for the jth system
        y_kapa       = States(i(1));
        c            = States(i(1) + 1 : i(2));
        ni_kapa      = States(i(3));
        sigma        = States(i(3) + 1 : i(4));
        theta_kapa   = States(i(4) + 1 : i(5));
        d            = States(i(5) + 1 : i(6));
        u_kapa       = States(i(6) + 1 : i(7));
        x            = States(i(7) + 1 : i(8));
        
        % Evaluate output and control signal
        y            = h{j}(x_all);
        u            = -kg(j) * theta_kapa(2:end) + u_kapa + d(1:2:end);
        
        phi          = [1; u];
        e            = y - y_kapa;
        sigma        = reshape(sigma, m(j) + 1, m(j) + 1);
        sigmaDot     = c * c' - kt(j) * sigma + Sigma(j) * eye(m(j) + 1);
        Phi          = [0 omega(j); -omega(j) 0];

        % Initialize output vector for the jth system
        StatesDot{j} = zeros(size(States'));

        % Compute time derivatives
        StatesDot{j}(i(1) + 1 : i(2)) = -K(j) * c + phi;
        StatesDot{j}(i(3))            = -K(j) * ni_kapa;
        StatesDot{j}(i(3) + 1 : i(4)) = sigmaDot(:);
        StatesDot{j}(i(4) + 1 : i(5)) = pinv(sigma) * (c * (e - ni_kapa) - Sigma(j) * theta_kapa);
        StatesDot{j}(i(5) + 1 : i(6)) = Phi * d;
        StatesDot{j}(i(6) + 1 : i(7)) = -1/tau(j) * theta_kapa(2:end);
        StatesDot{j}(i(7) + 1 : i(8)) = f{j}(x, u);
        StatesDot{j}(i(1))            = phi' * theta_kapa + K(j) * e + c'*StatesDot{j}(i(4) + 1 : i(5))' ;  %y kappa
    end
    
    % Convert cell array to matrix and transpose to match the expected output shape
    StatesDot       = cell2mat(StatesDot)';

end
