% Define system parameters
n = [1 1 1];               % Number of states for each system
m = [1 1 1];               % Number of inputs for each system
k = [0.5 0.5 0.5];         % Coefficients
a = [5 5 5];               % Sinusoid amplitude
N = 3;                     % Number of systems
time = 60;                 % Simulation time

% Initialize state and index cells
i = cell(1, N);
States = cell(1, N);
sistem_state_size = zeros(2, N);

% Define matrix A and function handles for system dynamics and output
l = 1;                     % Parameter for the matrix A
A = [3  l*2    l*1 ; 
     l*-2   3   l*1 ;
     l*-1   l*-1 3];       % Matrix A
f = {@(x,u) -x + u, @(x,u) -x + u, @(x,u) -x + u};   % Function handles for system dynamics
g = {@(x)A(1,1)/2 * (x(1) - 1)^2 + A(1,2) * x(2) * x(1) + A(1,3) * x(1) * x(3)...
   , @(x)A(2,2)/2 * (x(2) - 2)^2 + A(2,1) * x(2) * x(1) + A(2,3) * x(2) * x(3)...
   , @(x)A(3,3)/2 * (x(3) - 3)^2 + A(3,1) * x(3) * x(1) + A(3,2) * x(2) * x(3)};
% Define parameter cell array
param{1} = n;
param{2} = m; 
param{6} = k;
param{7} = f;
param{8} = g;
param{10} = N;

% Define base and delta omega
base_omega = [90 100 110];
delta_omega = [0];

% Start loop over delta omega
for koef = delta_omega
    omega_w = base_omega + koef;    % Omega width
    omega_h = omega_w * 2;          % Omega height
    omega_l = omega_w / 2;          % Omega length
    
    % Update parameter cell array
    param{3} = omega_l; 
    param{4} = omega_h;
    
    % Define Phi for each system
    for j = 1:N
        Phi{j} = [0 omega_w(j); -omega_w(j) 0];
    end
    param{5} = Phi;

    % Initialize states for each system
    States = cell(1, N);

    for j = 1 : N   
        i{j} = zeros(1, 8);

        % Define indices for different parts of the state vector
        i{j}(1) = 0;
        i{j}(2) = i{j}(1) + n(j);    
        i{j}(3) = i{j}(2) + m(j);
        i{j}(4) = i{j}(3) + m(j);
        i{j}(5) = i{j}(4) + 1;    
        i{j}(6) = i{j}(5) + 2 * m(j);

        % Define size of the system state
        sistem_state_size(:, j) = [i{j}(5); i{j}(6)];
        param{9} = sistem_state_size;

        % Initialize state for jth system
        States{j} = zeros(1, i{j}(6));
        States{j}(i{j}(1) + 1 : i{j}(2)) = zeros(1, n(j));
        States{j}(i{j}(2) + 1 : i{j}(3)) = zeros(1, m(j));
        States{j}(i{j}(3) + 1 : i{j}(4)) = zeros(1, m(j));
        States{j}(i{j}(4) + 1 : i{j}(5)) = zeros(1, m(j));
        States{j}(i{j}(5) + 1 : i{j}(6)) = repmat(a(j) * [sin(pi/3) cos(pi/3)], 1, m(j));
    end
    % Concatenate states from all systems
    States = cell2mat(States);
    [t,y_all] = ode45(@(t,y)dynamics_Frihauf(t, y, param),[0 time],States);
    sistem_index = [0, cumsum(sistem_state_size(2, :))];
    diff_omega_plot
end

% Solve system of linear equations
sol = linsolve(A, [3;6;9]);
hold on
% Plot solutions
for idx = 1:3
    subplot(1, 3, idx);
    plot(time, sol(idx), 'yo','Linewidth',4,'HandleVisibility','off');
end
beep
