% Declare given variables
n = [1 1 1];
m = [1 1 1];
alfa = 0.1;
N = 3;

% Initialization
i = cell(1, N);
States = cell(1, N);
sistem_state_size = zeros(2, N);

% Define constants
a = 0.1;
A = [3  a*2    a*1 ; 
     a*-2   3   a*1;
     a*-1     a*-1 3];

% Define function handles for the dynamics and output of the systems
f = {@(x,u) -x + u, @(x,u) -x + u, @(x,u) -x + u};
h = {@(x)A(1,1)/2 * (x(1) - 1)^2 + A(1,2) * x(2) * x(1) + A(1,3) * x(1) * x(3)...
   , @(x)A(2,2)/2 * (x(2) - 2)^2 + A(2,1) * x(2) * x(1) + A(2,3) * x(2) * x(3)...
   , @(x)A(3,3)/2 * (x(3) - 3)^2 + A(3,1) * x(3) * x(1) + A(3,2) * x(2) * x(3)};

x0 = [0.1333 2/3 1.2667];

% Loop over each system
for j = 1:N 
    eye_mat = eye(1 + m(j));
    
    % Define indices for different parts of the state vector
    i{j} = zeros(1, 8);
    i{j}(1) = 1;
    i{j}(2) = 2 + m(j);    
    i{j}(3) = 2 + m(j) + 1;
    i{j}(4) = 2 + m(j) + 2 + (1 + m(j)).^2 - 1;
    i{j}(5) = i{j}(4) + m(j) + 1;
    i{j}(6) = i{j}(5) + 2 * m(j);
    i{j}(7) = i{j}(6) + m(j);
    i{j}(8) = i{j}(7) + n(j);   
    
    y0 = h{j}(x0);
    sistem_state_size(:, j) = [i{j}(7); i{j}(8)];
    
    % Initialization of States
    States{j} = zeros(1, i{j}(8));
    States{j}(i{j}(1)) = 0;
    States{j}(i{j}(1) + 1 : i{j}(2)) = zeros(size(1, 1 + m(j)));
    States{j}(i{j}(3)) = y0 - States{j}(1);
    States{j}(i{j}(3) + 1 : i{j}(4)) = alfa * eye_mat(:);
    States{j}(i{j}(4) + 1 : i{j}(5)) = zeros(1, m(j) + 1);
    States{j}(i{j}(5) + 1 : i{j}(6)) = 0.5 * repmat([sin(pi/j) cos(pi/j)], 1, m(j));
    States{j}(i{j}(6) + 1 : i{j}(7)) = zeros(1, m(j));
    States{j}(i{j}(7) + 1 : i{j}(8)) = x0(j);
end

% Convert States cell array to matrix
States = cell2mat(States);

% Assign system parameters
param{1}  = n;                  % n
param{2}  = m;                  % m
param{3}  = [40, 50, 60];       % omega
param{4}  = 0.01;               % a
param{5}  = [1e-6, 1e-6, 1e-6]; % Sigma
param{6}  = [50, 50, 50];       % kt
param{7}  = [50, 50, 50];       % K
param{8}  = [0, 0, 0];          % kg
param{9}  = [15, 15, 15];       % tau
param{10} = f;
param{11} = h;
param{12} = sistem_state_size;
param{13} = N;

% Solve the system using ode45 solver
[t,y_all] = ode45(@(t,y)dynamics_INESC(t, y, param),[0 400],States);

% Define indices for systems
sistem_index = [0, cumsum(sistem_state_size(2, :))];

