% Function to execute a single step of the dynamical system
function [States] = dynamics_single_step(t, States, param, N)

    % Extract parameters from the param cell array
    h       = param{N + 1 + 1};  % Cost functions
    u_index = param{N + 1 + 2};  % Indices for control variables
    y_index = param{N + 1 + 3};  % Indices for output variables
    indices = param{N + 1 + 4};  % Indices for state variables
    x_index = param{N + 1 + 5};  % Indices for x states

    % Extract constraint parameters
    A_con   = param{N + 1}{1};   % Matrix A in the constraints
    b_con   = param{N + 1}{2};   % Vector b in the constraints
    ni      = param{N + 1}{3};   % Step size
    m       = param{N + 1}{4};   % Number of rows in A_con

    % Extract current state and control variables, and Lagrange multipliers
    x       = States(x_index);
    u       = States(u_index);
    lambda  = States(indices(N + 1) + 1 : end);

    % Calculate the product of A_con' and lambda, and reshape
    multipl = A_con' * lambda;
    multipl = reshape(multipl, m, []);

    % Update the output states based on the cost functions h(x)
    States(y_index) = h(x);    

    % Perform single step update for each player
    for i = 1:N
        States(indices(i) + 1 : indices(i + 1)) = single_step(t, States(indices(i) + 1 : indices(i + 1)), multipl(:, i), param{i});
    end
 
    % Update the Lagrange multipliers
    States(indices(N + 1) + 1 : end) = -lambda + proj_plus(lambda + ni * (A_con * (u + 2 * States(u_index)) - b_con));
    
end
