% Initialize parameters
sine_amps = [0.1 : 0.03 : 0.5];    % Sine amplitudes
faktors   = [10 : 10: 60];         % Factor omega
times     = [500000, 200000, 100000 * ones(1, 4), 50000 * ones(1, 5), 20000 * ones(1, 3)];
t_beg_vec = times * 0.8;           % Starting times for each sine amplitude

epsilon_e = [0.2 0.5 1.5];         % Epsilon error values
N         = 4;                     % Number of players

% Initialize result matrices
amplitudes = zeros(length(faktors), length(sine_amps), N);
times      = zeros(length(faktors), length(sine_amps), N, length(epsilon_e));

% Loop through factors and sine amplitudes
for p = 1:length(faktors)
    for l = 1:length(sine_amps) 
        
        % Extract values for this iteration
        sine_amp     = sine_amps(l);
        t_beg        = t_beg_vec(l);
        time         = times(l);
        faktor_omega = faktors(p);   
        
        % Load states for this iteration
        load(['States sine_amp=', num2str(sine_amp), ' faktor=', num2str(faktor_omega), '.mat'])    
    
        % Loop through each player
        for j = 1:N
            % Calculate mean of state x
            x_mean = mean(States_x{j}(t_beg:end, :));
            x_d    = vecnorm((States_x{j}(:, 1:2) - end_states(j, :))');
            
            % Compute amplitude and time for each epsilon
            amplitudes(p, l, j) = vecnorm((x_mean(:, 1:2) - end_states(j, :)));
            
            for eps = 1:length(epsilon_e)
                temp = find(x_d < epsilon_e(eps));
                if isempty(temp)
                    times(p, l, j, eps) = Inf;
                else
                    req_time = temp(find(bwareafilt([0, diff(temp)] == 1, 1), 1));
                    if isempty(req_time)
                        times(p, l, j, eps) = Inf;
                    else
                        times(p, l, j, eps) = req_time;
                    end
                end              
            end                        
        end        
    end
end

% Create a grid for contour plots
[SINE_AMPS, FAKTORS] = meshgrid(sine_amps, 10./faktors);

% Plot amplitude contours for player 4
figure
figure_standard
% Set the figure standard here, possibly with figure_standard function
[C, h] = contour(SINE_AMPS, FAKTORS, amplitudes(:, :, 4),'ShowText','on');
xlabel('Sine amplitude $d$');
ylabel('Frequency factor $k_\omega$');
clabel(C,h,'FontSize',8, 'interpreter', 'latex');

% Plot time contours for player 4 with epsilon = 1.5
figure
figure_standard
% Set the figure standard here, possibly with figure_standard function
[C, h] = contour(SINE_AMPS, FAKTORS, times(:, :, 4, 3)/3600, [2:2.5:17],'ShowText','on');
xlabel('Sine amplitude $d$');
ylabel('Frequency factor $k_\omega$');
clabel(C,h,'FontSize',8, 'interpreter', 'latex');
