% Function to perform a single step of the individual dynamics
function [StatesDot] = single_step(t, States, multiple, param)
    
    % Extract parameters from the param cell array
    m          = param{1};          % Number of inputs
    Phi        = param{2};          % 
    Sigma      = param{3};          % 
    kt         = param{4};          % 
    K          = param{5};          % 
    step       = param{6};          % Step size
    Omega_box  = param{9};          % Projection box for control action
    epsilon    = param{10};         % Small constant
    sys_dym    = param{11};         % System dynamics
    sys_par    = param{12};         % System parameters
    n          = param{13};         % State dimension

    % Define indices for different parts of the state vector
    i(1) = n;    
    i(2) = i(1) + 1;
    i(3) = i(2) + m + 1;    
    i(4) = i(3) + 1;
    i(5) = i(4) + (m + 1).^2;
    i(6) = i(5) + m + 1;
    i(7) = i(6) + 2 * m;
    i(8) = i(7) + m; 
    i(9) = i(8) + 1;

    % Extract state variables from the States vector
    x          = States(       1 : i(1));
    y_kapa     = States(i(1) + 1 : i(2));
    c          = States(i(2) + 1 : i(3));
    ni_kapa    = States(i(3) + 1 : i(4));
    sigma      = States(i(4) + 1 : i(5)); 
    theta_kapa = States(i(5) + 1 : i(6));
    d          = States(i(6) + 1 : i(7));
    u_kapa     = States(i(7) + 1 : i(8));
    y          = States(i(8) + 1 : i(9));
    
    % Calculate error
    e = y - y_kapa;
    
    % Update SigmaDot
    sigma    = reshape(sigma, m + 1, m + 1);
    sigmaDot = c * c' - kt * sigma + Sigma * eye(m + 1);

    % Initialize derivative of states
    StatesDot = zeros(size(States));
    
    % Calculate derivatives of the states
    StatesDot(       1 : i(1)) = sys_dym(t, x, u_kapa, sys_par);
    StatesDot(i(3) + 1 : i(4)) = epsilon * -K * ni_kapa;
    StatesDot(i(4) + 1 : i(5)) = epsilon * sigmaDot(:);
    StatesDot(i(5) + 1 : i(6)) = epsilon * sigma^(-1) * (c * (e - ni_kapa) - Sigma * theta_kapa);
    StatesDot(i(6) + 1 : i(7)) = epsilon * Phi * d;
    
    udot = -u_kapa + proj_box(u_kapa - step * (theta_kapa(2:end) + multiple) + d(1:2:end), Omega_box);
    StatesDot(i(7) + 1 : i(8)) = epsilon * udot;
    
    phi = [1; udot];
    StatesDot(i(2) + 1 : i(3)) = epsilon * (-K*c + phi);
    StatesDot(i(1) + 1 : i(2)) = epsilon * (phi' * theta_kapa + K * e + c'*StatesDot(i(5) + 1 : i(6)));
    StatesDot(i(8) + 1 : i(9)) = 0;
end
