function [StatesDot] = unicycle_dynamics(t, States, u, param)

    k1 = param{1};
    k2 = param{2};
    
    x = States(1);
    y = States(2);
    theta = States(3);
    
    x_ref = u(1);
    y_ref = u(2);
    
    v     = k1 * sqrt((x - x_ref)^2 + (y - y_ref)^2);
    phi   = -atan2(y_ref - y, x_ref - x) + theta;
    phi   = phi - floor((phi + pi)/(2 * pi)) * 2 * pi;
    omega = -k1 * sin(phi)* cos(phi) - k2 * phi;
    
    StatesDot = zeros(size(States));
    StatesDot(1) = v * cos(theta);
    StatesDot(2) = v * sin(theta);
    StatesDot(3) = omega;
    
end

