function [States] = dynamics_single_step(t, States, param, N)
    % Function to perform a single step of dynamics computation
    
    % Extract parameters from the input cell array 'param'
    h       = param{N + 1 + 1};    % Function handle for the output calculation
    u_index = param{N + 1 + 2};    % Indices of control variables in 'States'
    y_index = param{N + 1 + 3};    % Indices of output variables in 'States'
    indices = param{N + 1 + 4};    % Indices of variables for each player in 'States'
    x_index = param{N + 1 + 5};    % Indices of state variables in 'States'
    
    A_con   = param{N + 1}{1};     % Coefficient matrix for inequality constraints
    b_con   = param{N + 1}{2};     % RHS vector for inequality constraints
    ni      = param{N + 1}{3};     % Step size for projection
    m       = param{N + 1}{4};     % Dimension of control variables
    q       = param{N + 1}{5};     % Dimension of output variables
    
    x       = States(x_index);     % State variables
    u       = States(u_index);     % Control variables
    lambda  = States(indices(N + 1) + 1 : end);  % Multipliers
    
    multipl = A_con' * lambda;     % Intermediate calculation for multipliers
    multipl = reshape(multipl, m, []);  % Reshape multipliers matrix
    
    States(y_index) = h(x);    % Compute the output variables using function handle 'h'
    
    % Perform dynamics computation for each player
    for i = 1:N
        States(indices(i) + 1 : indices(i + 1)) = single_step(t, States(indices(i) + 1 : indices(i + 1)), multipl(:, i), param{i});
    end
    
    % Perform projection for the coordinator player
    States(indices(N + 1) + 1 : end) = - lambda + proj_plus(lambda + ni * (A_con * (u + 2 * States(u_index)) - b_con));
end
