% Clear all variables except 'end_states'
clearvars -except end_states 

n = 3;  % Dimensionality of the state space
m = 2;  % Number of inputs 
q = 1;  % Number of constraints, rows of A_con
alfa = 0.5;  % Alpha parameter used in the cost function
t_samp = 1;  % Sampling time
N = 4;  % Number of iterations

% Construction of the cost function
a = 0.08;
Q_proto = -eye(m) * a;  % Basic Q matrix for quadratic cost
Q = repmat(Q_proto, N, N);  % Replicate the basic Q matrix for each iteration
Q(eye(m * N) == 1) = (1 + (N - 1) * a/2);  % Modify diagonal of Q to implement a special rule

R_s = [-4, -8; -12, -3; 1, 7; 16, 8]';  % Reward vector for each state
R = R_s(:);  % Convert the reward matrix to a column vector

% Preallocation and generation of cell array with identity matrices at certain positions
cell_eye = cell(1, N);
for i = 1:N
    cell_eye{i} = [zeros(m * (i - 1), m * N); zeros(m, m * (i - 1)), eye(m), zeros(m, m * (N - i)); zeros(m * (N - i), m * N)];
    cell_eye{i} = cell_eye{i} == 1;
end

% Construction of the constraint matrices
a = 14; 
A_proto = [repmat(eye(m), N -1, 1), -eye(m * (N - 1))];
A_proto = [A_proto; -A_proto];
A_con = A_proto;
for i = 2:N
    A_temp = A_proto;
    A_temp(:, [1:m, m * (i - 1) + (1:m)]) = A_temp(:, [m * (i - 1) + (1:m), 1:m]);
    A_con = [A_con; A_temp];
end

b_con = a * ones(size(A_con, 1), 1);  % Right-hand side of the constraints
q = size(A_con, 1);  % Update number of constraints

% Various parameters for the dynamics and constraints
K          = 100;
kt         = 100;
Sigma      = 1e-6;
step_size  = 0.005;
Omega_box  = {[-16 16; -6 6], [-16 16; -6 6], [-16 16; -6 6], [-16 16; -6 6], [-16 16; -6 6], [-16 16; -6 6]};
epsilon    = 0.2;
system_params = {3, 6};
faktor_omega = 10;
sine_amp   = 0.50;

colors = [[0, 0.4470, 0.7410]; [0.8500, 0.3250, 0.0980]; [0.9290, 0.6940, 0.1250]; [0.4940, 0.1840, 0.5560]; [0.4660, 0.6740, 0.1880]; [0.3010, 0.7450, 0.9330]];

more_info_string = [...
'\epsilon = ', num2str(epsilon), ...
', k_1 = ', num2str(system_params{1}), ...
', k_2 = ',num2str(system_params{2}), ...
', \omega_k = ', num2str(faktor_omega), ...
', \gamma = ', num2str(step_size), newline];

sine_amps    = [0.49];  % Amplitudes of sine wave input
faktors      = [10];  % Factors for omega
times        = [10000];  % Simulation time
t_beg_vec    = times * 0.8;  % Initial time

noise_level = [1 3 5];  % Levels of noise
broj_ponavljanja = 10;  % Number of repetitions

A_ll.mean    = cell(1, N);
A_ll.max     = cell(1, N);
A_ll.min     = cell(1, N);
A_ll.amp     = cell(1, N);
A_ll.index   = cell(1, N);
A_ll.close   = cell(1, N);
A_ll.d       = cell(1, N);

for noise_i = 1 : size(noise_level, 2)
    for broj = 1:broj_ponavljanja
        
        p = 1;
        l = 1;
        rng('shuffle')
        noise = randn(4, times + 1) * noise_level(1, noise_i);
        sine_amp = sine_amps(1, l);
        t_beg    = t_beg_vec(1, l);
        time     = times(1, l);
        faktor_omega = faktors(1, p);

        more_info_string = [more_info_string, 'sine amp = ', mat2str(sine_amp)];

        i               = cell(1, N + 1);
        States          = cell(1, N + 1);
        sistem_index    = zeros(1, N + 1);
        x_index         = [];
        u_index         = [];
        y_index         = [];
        previous_index  = zeros(1, N + 1);

        rng(13);

        for j = 1:N 

            eye_mat = eye(m + 1);

            i{j}(1) = n;
            i{j}(2) = i{j}(1) + 1;
            i{j}(3) = i{j}(2) + m + 1;    
            i{j}(4) = i{j}(3) + 1;
            i{j}(5) = i{j}(4) + (m + 1).^2;
            i{j}(6) = i{j}(5) + m + 1;
            i{j}(7) = i{j}(6) + 2 * m;
            i{j}(8) = i{j}(7) + m; 
            i{j}(9) = i{j}(8) + 1;

            States{j} = zeros(1, i{j}(8));

            States{j}(          1 : i{j}(1)) = zeros(1, n);
            States{j}(i{j}(1) + 1 : i{j}(2)) = 2;  % y kappa
            States{j}(i{j}(2) + 1 : i{j}(3)) = [1/K]; % zeros(size(1, 1 + m)); % c
            States{j}(i{j}(3) + 1 : i{j}(4)) = 0; % y0 - States(1); % ni kappa, u ovome slucaju ni_kappa = ni, a ni = e
            States{j}(i{j}(4) + 1 : i{j}(5)) = alfa * eye_mat(:); % sigma % [(1/K^2 + Sigma)/kt 0; 0 Sigma/kt]; %
            States{j}(i{j}(5) + 1 : i{j}(6)) = zeros(1, m + 1); % theta kappa [2/3]; %
            States{j}(i{j}(6) + 1 : i{j}(7)) = sine_amp * repmat([sin(pi/3) cos(pi/3)], 1, m); % d
            States{j}(i{j}(7) + 1 : i{j}(8)) = zeros(1, m); % u kappa 1/3; %
            States{j}(i{j}(8) + 1 : i{j}(9)) = 0;

            x_index                 = [x_index, (1 : i{j}(1)) + previous_index(j)];
            u_index                 = [u_index, (i{j}(7) + 1 : i{j}(8)) + previous_index(j)];
            y_index                 = [y_index, i{j}(9) + previous_index(j)];
            previous_index(j + 1)   = previous_index(j) + i{j}(9);

            param{j}     =  cell(1, 6);
            param{j}{1}  =  m;           
            param{j}{2}  =  [30 + 10 * (2 * j - 1) , 30 + 10 * (2 * j)] * (0.5 + rand(1))/faktor_omega;   % omega
            param{j}{3}  =  Sigma;       
            param{j}{4}  =  kt;          
            param{j}{5}  =  K;           
            param{j}{6}  =  step_size;   
            param{j}{7}  =  A_con;
            param{j}{8}  =  q;
            param{j}{9}  =  Omega_box{j};
            param{j}{10} =  epsilon;
            param{j}{11} =  @unicycle_dynamics;
            param{j}{12} =  system_params;
            param{j}{13} =  n;
            param{j}{14} =  noise(j, :);

            Omega_cell = cell(1, m);
            for op = 1:m
                Omega_cell{op} = [0 param{j}{2}(op); -param{j}{2}(op) 0];
            end

            Phi = diag_matrices(Omega_cell);
            param{j}{2} = Phi;

        end

        %% N + 1 Player is the coordinator
        States{N + 1}    = zeros(1, q);
        param{N + 1}     = cell(1, 3);
        param{N + 1}{1}  = A_con;
        param{N + 1}{2}  = b_con;
        param{N + 1}{3}  = step_size;
        param{N + 1}{4}  = m;
        param{N + 1}{5}  = q;
        States = cell2mat(States);

        %%
        indices     = 1:(N * n);
        indices_all = indices(mod(indices, 3) ~= 0);
        indices_j   = reshape(indices_all, n - 1, [])';
        indices_minus_j = zeros((n - 1) * (N - 1), N);
        for j = 1:N
            temp = indices_j';
            temp(:, j) = [];
            indices_minus_j(:, j) = temp(:);
        end
        indices_minus_j = indices_minus_j';

        indices_reg = reshape(1 : (n - 1) * N, n - 1, [])';

        %%
        param{N + 1 + 1}  =  @(x)[x(indices_j(1, :))' * Q(indices_reg(1, :), :) * x(indices_all) + (R(indices_reg(1, :)) - 2 * x(indices_j(1, :)))' *  R(indices_reg(1, :)) + a/2 * x(indices_minus_j(1, :))' * x(indices_minus_j(1, :)); ...
                                  x(indices_j(2, :))' * Q(indices_reg(2, :), :) * x(indices_all) + (R(indices_reg(2, :)) - 2 * x(indices_j(2, :)))' *  R(indices_reg(2, :)) + a/2 * x(indices_minus_j(2, :))' * x(indices_minus_j(2, :)); ...
                                  x(indices_j(3, :))' * Q(indices_reg(3, :), :) * x(indices_all) + (R(indices_reg(3, :)) - 2 * x(indices_j(3, :)))' *  R(indices_reg(3, :)) + a/2 * x(indices_minus_j(3, :))' * x(indices_minus_j(3, :)); ...
                                  x(indices_j(4, :))' * Q(indices_reg(4, :), :) * x(indices_all) + (R(indices_reg(4, :)) - 2 * x(indices_j(4, :)))' *  R(indices_reg(4, :)) + a/2 * x(indices_minus_j(4, :))' * x(indices_minus_j(4, :))]; % ...
                                  % x(indices_j(5, :))' * Q(indices_reg(5, :), :) * x(indices_all) + (R(indices_reg(5, :)) - 2 * x(indices_j(5, :)))' *  R(indices_reg(5, :)) + a/2 * x(indices_minus_j(5, :))' * x(indices_minus_j(5, :)); ...
                                  % x(indices_j(6, :))' * Q(indices_reg(6, :), :) * x(indices_all) + (R(indices_reg(6, :)) - 2 * x(indices_j(6, :)))' *  R(indices_reg(6, :)) + a/2 * x(indices_minus_j(6, :))' * x(indices_minus_j(6, :))]; % vector of cost functions

        param{N + 1 + 2}  =  u_index;
        param{N + 1 + 3}  =  y_index;
        param{N + 1 + 4}  =  previous_index;
        param{N + 1 + 5}  =  x_index;

        tic
        [t,y] = ode45(@(t,y)dynamics_single_step(t, y, param, N),[0:t_samp:time],States);
        toc
        beep


        %%

        States_x = cell(1, N);
        States_u = cell(1, N);

        for j = 1:N
            States_x{j} = y(:, (1 : i{j}(1)) + previous_index(j));
        end
        figure
        figure_standard
        hold on
        for j = 1:N
            plot(States_x{j}(:, 1), States_x{j}(:, 2), 'DisplayName', ['Player ', int2str(j)], 'Color', colors(j, :))
        end
        ime = ['noise_', 'sigma=', num2str(noise_level(1, noise_i)),'_', num2str(broj)];
        xlim([-20 20])
        ylim([-8 8])
        saveas(gcf,[ime, '.png'])
        savefig([ime, '.fig'])
        pause(2)
        close all
    end
end
