function [C] = power_coef(x, k, D, k_d)
    %% Input arguments 
    % x are the positions of the windturbines
    % k is the vector of the wind direction
    % D is the windturbine diameter
    % k_d coeficient that describes the spread of the wind wake
    
    %% Code
    N = size(x, 2);
    
    C = zeros(N, N);
    
    for i = 1:N
        for j = i + 1:N
            
            vert_d = distance(x(:, j), x(:, i), k);
            D_new  = D + 2 * k_d * vert_d;
            horz_d = distance(x(:, j), x(:, i), [k(2); -k(1)]);
            A      = overlap(horz_d, D/2, D_new/2);            
            c      = 4 * A / (D_new^2 * pi);
            
            if x(:, j)' * k < x(:, i)' * k %j in front of i turbine
                C(j, i) = 0;
                C(i, j) = c;            
            else
                C(i, j) = 0;
                C(j, i) = c;
            end
        end
    end
    
end

