% Clear all variables except vGNE
load vGNE.mat
clearvars -except vGNE 

% Initialize parameters
n = 1;
m = 1;
alfa = 0.5;
t_samp = 1;
save_variables = false;

% Geometry of wind farms
i_r      = 5;                % rows in grid of wind farms
i_c      = 5;                % columns in grid of wind farms
N        = i_r * i_c;        % total number of wind turbines
step_y   = 560;              % step size in y direction
k_dir    = -5;               

% Compute x positions of wind farms
x_pos    = [];               
for j = ceil(i_c/2) - i_c + (i_r -1:-1:0)
    for i = ceil(i_r/2) - i_r + (0:i_r -1)
        x_pos = [x_pos, [step_y * j/k_dir + i * step_y; step_y * j]];
    end
end

% Set up additional wind farm parameters
D_turb   = 80;               % turbine diameter
A_turb   = D_turb^2 * pi / 4;% turbine area
rho      = 1.225;            % air density
k_d      = 0.04;             % wake decay constant
U_infy   = 8;                % free stream wind speed

% Constructing the constraint matrices
A_con = zeros(i_c * i_c * (i_r - 1), i_r * i_c);   % Initialize constraint matrix A_con
for i = 0:i_c-1
    for j = 0:i_r - 2
        A_con(j * i_c * i_c + ((i_c * i + 1) : ((i + 1) * i_c)), i_c * j + i + 1) = 1;
        A_con(j * i_c * i_c + ((i_c * i + 1) : ((i + 1) * i_c)), i_c * (j + 1) + (1 : i_c)) = -eye(i_c);
    end
end
A_con = [A_con; -A_con];     % Concatenate A_con and its negative

% Set up constraint vector b_con
q = size(A_con, 1);          % number of constraints, rows of A_con
b_con = 0.03 * ones(q, 1);   % Initialize constraint vector b_con

% Scale constraints for normalization
A_con = A_con * 10;
b_con = b_con * 10;

% Initialize other parameters
K       = 100;
kt      = 100;
Sigma   = 1e-6;
[Omega_box{1:N}] = deal([0.1 1/3]);  % box constraints for Omega
step_size       = 0.05;
epsilon         = 0.005;
system_params   = 0.1;
faktor_omega    = 100;
sine_amp        = 0.01;
reduction       = 10^4;

k_nuliranje  = 1;
times        = 25000;

% Define RGB color codes for plots and additional information
colors = [[0, 0.4470, 0.7410]; [0.8500, 0.3250, 0.0980]; [0.9290, 0.6940, 0.1250]; [0.4940, 0.1840, 0.5560]; [0.4660, 0.6740, 0.1880]; [0.3010, 0.7450, 0.9330]];
more_info_string = [...
'\epsilon = ', num2str(epsilon), ...
', \omega_k = ', num2str(faktor_omega), ...
', \gamma = ', num2str(step_size), newline ...
', sine amp = ', mat2str(sine_amp)];


k_winds      = [2 0 -1; -1 -1 -1];          % Different wind derections
Solution     = cell(1, size(k_winds, 2));   % Solution cell
t_last       = 0;

% Loop through function shapes and amplitudes
sine_amps    = sine_amp;
funkcije     = {@sin, @cos, @square, @sawtooth};
for p = 1 : size(k_winds, 2)
    for l = 1 : size(sine_amps, 2) 
        
        sine_amp = sine_amps(1, l)
        time     = times(1, l);
        k_wind   = k_winds(:, p)

        % Compute the total produced power
        C        = power_coef(x_pos, k_wind, D_turb, k_d);
        J        = @(x)-cost_sum(x, C, U_infy, A_turb, rho)/reduction;
        [end_states, J_end] = fminunc(@(x)-sum(cost(x, C, U_infy, A_turb, rho)), ones(N, 1)/3);

        if p == 1
            % Initialize data structures only the first time
            i               = cell(1, N + 1);
            States          = cell(1, N + 1);
            sistem_index    = zeros(1, N + 1);
            x_index         = [];
            u_index         = [];
            y_index         = [];
            previous_index  = zeros(1, N + 1);

            % Seed for random number generation
            rng(13);

            % Construct State vector
            for j = 1:N 

                eye_mat = eye(m + 1);

                i{j}(1) = n;
                i{j}(2) = i{j}(1) + 1;
                i{j}(3) = i{j}(2) + m + 1;    
                i{j}(4) = i{j}(3) + 1;
                i{j}(5) = i{j}(4) + (m + 1).^2;
                i{j}(6) = i{j}(5) + m + 1;
                i{j}(7) = i{j}(6) + 2 * m;
                i{j}(8) = i{j}(7) + m; 
                i{j}(9) = i{j}(8) + 1;

                States{j} = zeros(1, i{j}(8));

                States{j}(          1 : i{j}(1)) = ones(1, n)/3;
                States{j}(i{j}(1) + 1 : i{j}(2)) = 2;  %y kappa
                States{j}(i{j}(2) + 1 : i{j}(3)) = [1/K];%zeros(size(1, 1 + m)); %c
                States{j}(i{j}(3) + 1 : i{j}(4)) = 0;%y0 - States(1);%ni kappa, u ovome slucaju ni_kappa = ni, a ni = e
                States{j}(i{j}(4) + 1 : i{j}(5)) = alfa * eye_mat(:); %sigma %[(1/K^2 + Sigma)/kt 0; 0 Sigma/kt];%
                States{j}(i{j}(5) + 1 : i{j}(6)) = zeros(1, m + 1); %theta kappa [2/3];%
                States{j}(i{j}(6) + 1 : i{j}(7)) = k_nuliranje * sine_amp * repmat([sin(pi/3) cos(pi/3)], 1, m); %d
                States{j}(i{j}(7) + 1 : i{j}(8)) = ones(1, m)/3; %u kappa 1/3;%
                States{j}(i{j}(8) + 1 : i{j}(9)) = 0;

                % Update indices
                x_index                 = [x_index, (1 : i{j}(1)) + previous_index(j)];
                u_index                 = [u_index, (i{j}(7) + 1 : i{j}(8)) + previous_index(j)];
                y_index                 = [y_index, i{j}(9) + previous_index(j)];
                previous_index(j + 1)   = previous_index(j) + i{j}(9);

                % Construct parameters for each agent
                param{j}     =  cell(1, 6);
                param{j}{1}  =  m;           % m
                param{j}{2}  =  3 + [30 * j + 10 * (0.5 + rand(1))] /faktor_omega;
                param{j}{3}  =  Sigma;       % Sigma
                param{j}{4}  =  kt;          % kt
                param{j}{5}  =  K;           % K
                param{j}{6}  =  step_size;   % tau
                param{j}{7}  =  A_con;
                param{j}{8}  =  q;
                param{j}{9}  =  Omega_box{j};
                param{j}{10} =  epsilon;
                param{j}{11} =  @induction_dynamics;
                param{j}{12} =  system_params;
                param{j}{13} =  n;
                param{j}{14} =  funkcije{1 + mod(j, 4)};
                
                % Generate diagonal matrices
                Omega_cell = cell(1, m);
                for op = 1:m
                    Omega_cell{op} = [0 param{j}{2}(op); -param{j}{2}(op) 0];
                end

                Phi = diag_matrices(Omega_cell);
                param{j}{2} = Phi;

            end

            %% N + 1 Player is the coordinator
            States{N + 1}    = zeros(1, q);
            param{N + 1}     = cell(1, 3);
            param{N + 1}{1}  = A_con;
            param{N + 1}{2}  = b_con;
            param{N + 1}{3}  = step_size;
            param{N + 1}{4}  = m;
            param{N + 1}{5}  = q;
            States = cell2mat(States);

            param{N + 1 + 1}  =  J;
            param{N + 1 + 2}  =  u_index;
            param{N + 1 + 3}  =  y_index;
            param{N + 1 + 4}  =  previous_index;
            param{N + 1 + 5}  =  x_index;
        else
            States              = y(end, :);
            param{N + 1 + 1}    = J;
            t_last              = t(end);
        end
        tic
        [t,y] = ode45(@(t,y)dynamics_single_step(t, y, param, N), [0:t_samp:time], States);
        toc
        beep
        %%

        t = t_last + t;
        
        % Extract the states
        States_x = cell(1, N);
        for j = 1:N
            States_x{j} = y(:, (1 : i{j}(1)) + previous_index(j));
        end
        States_a = cell2mat(States_x)';
        
        P_all = cost_sum(States_a, C, U_infy, A_turb, rho);
        
        Solution{p}.x = States_x;
        Solution{p}.a = States_a;
        Solution{p}.P = P_all;
        Solution{p}.t = t;
        Solution{p}.C = C;
        Solution{p}.J = -J_end;

    
    end
end

%%
t       = [];
P       = [];
P_opt   = [];
t_opt   = [];
P_sel   = [];
t_sel   = [];

for p = 1:size(k_winds, 2)
    P       = [P Solution{p}.P];
    t       = [t Solution{p}.t'];
    P_opt   = [P_opt ones(1, 2) * Solution{p}.J];
    t_opt   = [t_opt Solution{p}.t(1) Solution{p}.t(end)];
    P_sel   = [P_sel ones(1, 2) * cost_sum(ones(N, 1) /3, Solution{p}.C, U_infy, A_turb, rho)];
    t_sel   = [t_sel Solution{p}.t(1) Solution{p}.t(end)];    
end


t_gne = [0 Solution{1}.t(end) Solution{1}.t(end) Solution{2}.t(end) ...
    Solution{2}.t(end) Solution{3}.t(end)];
P_gne = [vGNE(1) vGNE(1) vGNE(2) vGNE(2) vGNE(3) vGNE(3)];

figure
figure_standard
hold on
plot(t/3600, P/10^6, 'DisplayName', 'Power output')
plot(t_sel/3600, P_sel/10^6, 'r--', 'DisplayName', 'Greedy power output')
plot(t_gne/3600, P_gne/10^6, 'm-.', 'DisplayName', 'v-GNE power output')
legend()
title(['Power output'])%, newline, more_info_string])
xlabel('$t [h]$')
ylabel('$P [GW]$')




