function inside = D_gnep(States)
    % Description: Jump set
    % Return 0 if outside of D, and 1 if inside D
    
    % Global parameter for input
    global param_input;
    
    % Input parameters
    g           = param_input{4};       % Vector of functions for convex constraints g(x)<= 0
    k_limit     = param_input{6};       % Min and max values of the dual coefficients
    i           = param_input{8};       % Indices used in iterations
    epsilon     = param_input{9};       % Parameter used for creating hysteresis
    
    % System states
    u       = States(i(1) + 1 : i(2));
    k       = States(i(5) + 1 : i(6));
    s       = States(i(6) + 1 : i(7));
    
    % Evaluate g function
    g_eval  = g(u);
    
    % Determine if inside the jump set
    change_index = (g_eval >= 2 * epsilon & s == 0) | ...
        (g_eval <= epsilon & s == 1) | (k >= k_limit(2) & (s == 1 | s == 0));
    
    inside = sum(change_index) >= 1;
end
