function StatesDot = f_gnep(States)
    % Global parameters for inputs and internal
    global param_input param_internal last_time;
    
    % Internal parameters
    n               = param_internal{9};    % Vector containing the sum of u sizes
    J               = param_internal{8};    % Vector of cost functions
    epsilon_e       = param_internal{1};    % Timescale separation parameters (oscillator dynamics / system)
    epsilon_f       = param_internal{2};
    epsilon_e_dual  = param_internal{3};
    epsilon_f_dual  = param_internal{4};

    % Input parameters
    m               = param_input{3};       % Number of convex constraints
    g               = param_input{4};       % Vector of functions for convex constraints g(x)<= 0
    nabla_g         = param_input{5};       % Gradient function for the convex constraints
    k_dot           = param_input{7};       % Speed at which the k increases
    i               = param_input{8};       % Indices used in iterations
    kappa           = param_input{10};      % Vector containing the perturbation frequencies 
    F               = param_input{11};      % Pseudo gradient as function of known and unknown variables
    
    % System states
    u               = States(i(1) + 1 : i(2));
    a               = States(i(1) + sum(n)/2 + 1 : i(2));
    z               = States(i(2) + 1 : i(3));
    w               = States(i(3) + 1 : i(4));
    lambda          = States(i(4) + 1 : i(5));
    k               = States(i(5) + 1 : i(6));
    s               = States(i(6) + 1 : i(7));
    zeta            = States(i(7) + 1 : i(8));
    t               = States(i(8) + 1 : i(9));
    
    % Calculate mi
    mi  = sin(2 * pi * kappa * t + [pi/4  -3*pi/4 3*pi/4 -pi/4 ]');
    
    % Display simulation time and real time for every 100 units of simulation time
    if (mod(t, 100) < 1 && last_time < floor(t))
        last_time = floor(t);
        [c , ~] = clock;
        display(['Simulation Time: ', num2str(t)])
        display(['Real Time: ', num2str(c(4:6))])
    end
    
    % Initialize state derivatives
    StatesDot = zeros(size(States));
    
    % Compute state derivatives   
    StatesDot(i(1) + 1 : i(2)) = epsilon_f .* epsilon_e .* (-u + z - 10 * (F(zeta, a) + nabla_g(u) * lambda)) ;
    StatesDot(i(2) + 1 : i(3)) = epsilon_f .* epsilon_e .* (-z + u);
    StatesDot(i(3) + 1 : i(4)) = epsilon_f_dual .* epsilon_e_dual .* (-w + lambda);
    StatesDot(i(4) + 1 : i(5)) = epsilon_f_dual .* epsilon_e_dual .* k .* (lambda .* (g(u) - lambda + w)); 
    StatesDot(i(5) + 1 : i(6)) = epsilon_f_dual .* epsilon_e_dual .* sin(pi/2 * s) .* k_dot;
    StatesDot(i(7) + 1 : i(8)) = epsilon_e(1:n/2) .* (-zeta + 2./a .* (J(u(1 : n/2) + a .* mi).* mi));
    StatesDot(i(8) + 1 : i(9)) = 1;

end