function StatePlus = g_gnep(States)
    % Global parameter for input
    global param_input;
    
    % Input parameters
    g           = param_input{4};       % Vector of functions for convex constraints g(x)<= 0
    k_limit     = param_input{6};       % Min and max values of the dual coefficients
    i           = param_input{8};       % Indices used in iterations
    epsilon     = param_input{9};       % Parameter used for creating hysteresis
    
    % System states
    u       = States(i(1) + 1 : i(2));
    k       = States(i(5) + 1 : i(6));
    s       = States(i(6) + 1 : i(7));
    
    % Initialize StatePlus
    StatePlus = States;
    
    % Evaluate g function
    g_eval  = g(u);
    
    % Determine state transitions
    change_index = (g_eval >= 2 * epsilon & s == 0) | ...
        (g_eval <= epsilon & s == 1);
    
    % Update s based on conditions
    s(change_index) = 1 - s(change_index);
    change_index = (k >= k_limit(2) & (s == 1 | s == 0));
    s(change_index) = 2;

    % Update StatePlus with new s values
    StatePlus(i(6) + 1 : i(7)) = s;
end
