function [t, j, x] = hybrid_gnep_solver(f_gnep, g_gnep, C_gnep, D_gnep,...
    States_init, TSPAN, JSPAN, rule, options, solver)

    global param_input param_internal;


    % Vectorization of the amplitudes and the epsilons
    n           = param_input{1}; 
    J           = param_input{2};
    m           = param_input{3}; 
    epsilon_e   = param_input{12};
    epsilon_f   = param_input{13};
    
    epsilon_primal = [];
    eps_f_primal   = [];
    for j = 1:size(n, 2)
        epsilon_primal = [epsilon_primal; ones(n(j), 1) * epsilon_e(j)];
        eps_f_primal   = [eps_f_primal; ones(n(j), 1) * epsilon_f(j)];
    end
    param_internal{1} = epsilon_primal;
    param_internal{2} = eps_f_primal;
    
    epsilon_dual = ones(m, 1) * epsilon_e(end);
    eps_f_dual = ones(m, 1) * epsilon_f(end);
    
    param_internal{3} = epsilon_dual;
    param_internal{4} = eps_f_dual;
    
    epsilon_sine = ones(sum(n) / 2, 1) * epsilon_e(end);
    eps_f_sine = ones(sum(n) / 2, 1) * epsilon_f(end);
    
    param_internal{5} = epsilon_sine;
    param_internal{6} = eps_f_sine;
    
   
    % Vectorization of the cost functions
    % Note: It might be possible to recursively do this, but I am afraid
    % this might be slow and it might be problematic. Using the str2func
    % function seems to be the most elegant approach.
    
    command_to_eval = '@(u)[';
    for j = 1:size(n, 2)/2
        f_string = func2str(J{j});
        for l = 1:n(j)            
            command_to_eval = [command_to_eval, f_string(5:end),';'];
        end
    end
    command_to_eval(end) = ']';
    command_to_eval(end + 1) = ';';    
    param_internal{8} = str2func(command_to_eval);       
    
    % Summation of n. The algorithm only needs to know the complete size of
    % the whole system.
    param_internal{9} = sum(n);


[t,j,x] = HyEQsolver(f_gnep, g_gnep, C_gnep, D_gnep,...
    States_init, TSPAN, JSPAN, rule, options, solver);
end

