% Define the vector of objective functions
J_f = @(u) [J{1}(u); J{2}(u); J{3}(u); J{4}(u)];

% Start index for slicing arrays
s = 1; 

% Extract u and a for both dither and no dither cases
u_dither    = x_dither(s:end, i(1) + 1:i(1) + 4)';      
u_no_dither = x_no_dither(s:end, i(1) + 1:i(1) + 4)';   
a_dither    = x_dither(s:end, i(1) + sum(n)/2 + 1:i(2))';

% Define mi for both dither and no dither cases
mi_dither   = sin(2 * pi * kappa * t_dither(s:end, 1)' + [pi/4, -3*pi/4, 3*pi/4, -pi/4]');
mi_no_dither = sin(2 * pi * kappa * t_no_dither(s:end, 1)' + [pi/4, -3*pi/4, 3*pi/4, -pi/4]');

% Calculate objective function values for both dither and no dither cases
J_eval_dither    = J_f(u_dither + a_dither .* mi_dither);
J_eval_no_dither = J_f(u_no_dither + amin * mi_no_dither);

% Define the sum objective function
f = @(u)J{1}(u) + J{2}(u) + J{3}(u) + J{4}(u);

% Use fmincon to optimize the objective function
x_opt = fmincon(f, x_no_dither(end, 1: 4)', [1 2 3 4], Umax);

%%
%Plot of sum of function evaluations
figure(5)
hold on

% Define the sampling rate and time span for the plot
sample_T = 1;
begin_t  = 0 * 3600;    % Begin time in seconds
end_t    = 2 * 3600;    % End time in seconds

% Get the indices for the desired time span for dither case
i_dither_begin = find(t_dither >= begin_t); i_dither_begin = i_dither_begin(1);
i_dither_end   = find(t_dither >= end_t); i_dither_end = i_dither_end(1);
span_dither    = i_dither_begin : sample_T : i_dither_end;

% Get the indices for the desired time span for no dither case
i_no_dither_begin = find(t_no_dither >= begin_t); i_no_dither_begin = i_no_dither_begin(1);
i_no_dither_end   = find(t_no_dither >= end_t); i_no_dither_end = i_no_dither_end(1);
span_no_dither    = i_no_dither_begin : sample_T : i_no_dither_end;

% Plot the objective function values for the specified time span in hours
plot(t_no_dither(span_no_dither) / 3600, 100 * (-sum(J_eval_no_dither(:, span_no_dither), 1) + f(x_opt)) / -f(x_opt), 'r')
plot(t_dither(span_dither) / 3600, 100 * (-sum(J_eval_dither(:, span_dither), 1) + f(x_opt)) / -f(x_opt), 'b')

% Apply standard figure formatting
figure_standard

% Label the axes and add a legend
xlabel('t [h]', 'interpreter', 'latex')
ylabel('$\sum f_i(\hat{x}_i) - \sum f_i(\hat{x}^*_i)$', 'interpreter', 'latex')
xlim([begin_t, end_t] / 3600)
legend("W/o amplitude optimization", "With amplitude optimization")

% Show grid lines on the plot
grid on

% Create a subplot with specified position within the figure
lower_plot = axes('Position',[.4 .2 .45 .4]); 
box on
hold on

% Set sampling rate and time span for this plot
sample_T = 1;
begin_t  = 2 * 3600;     % Begin time in seconds
end_t    = 9.5 * 3600;   % End time in seconds

% Get indices for the desired time span for the dither case
i_dither_begin = find(t_dither>=begin_t); 
i_dither_begin = i_dither_begin(1);
i_dither_end   = find(t_dither>=end_t); 
i_dither_end   = i_dither_end(1);
span_dither    = i_dither_begin : sample_T : i_dither_end;

% Get indices for the desired time span for the no dither case
i_no_dither_begin = find(t_no_dither>=begin_t); 
i_no_dither_begin = i_no_dither_begin(1);
i_no_dither_end   = find(t_no_dither>=end_t); 
i_no_dither_end   = i_no_dither_end(1);
span_no_dither    = i_no_dither_begin : sample_T : i_no_dither_end;

% Plot the objective function values for the specified time span in hours
plot(t_no_dither(span_no_dither)/3600, 100 * (-sum(J_eval_no_dither(:, span_no_dither), 1) + f(x_opt))./1, 'r')
plot(t_dither(span_dither)/3600, 100 * (-sum(J_eval_dither(:, span_dither), 1) + f(x_opt))./1, 'b')

% Create a new figure
figure;

% Hold the current plot to allow subsequent plotting commands to add to the existing graph
hold on

% Set sampling rate and time span for this plot
sample_T = 1;                           % Sampling rate
begin_t  = 0 * 3600;                     % Begin time in seconds
end_t    = 1 * 3600;                     % End time in seconds

% Get indices for the desired time span for the dither case
i_dither_begin = find(t_dither >= begin_t); 
i_dither_begin = i_dither_begin(1);
i_dither_end   = find(t_dither >= end_t); 
i_dither_end   = i_dither_end(1);
span_dither    = i_dither_begin : sample_T : i_dither_end;

% Plot for the dither case for variables a_1 to a_4 
plot(t_dither(span_dither, :)/3600, x_dither(span_dither, 5:8));
grid on
xlabel('t [h]', 'interpreter', 'latex')
ylabel('$a_i$', 'interpreter', 'latex')
legend('$a_1$', '$a_2$', '$a_3$', '$a_4$')
figure_standard
xlim([begin_t end_t]/3600)

% Create a subplot with specified position within the figure
lower_plot = axes('Position',[.4 .6 .45 .3]); 
box on
hold on

% Set sampling rate and time span for this plot
sample_T = 1;                           % Sampling rate
begin_t  = 1 * 3600;                     % Begin time in seconds
end_t    = 6 * 3600;                     % End time in seconds

% Get indices for the desired time span for the dither case
i_dither_begin = find(t_dither >= begin_t); 
i_dither_begin = i_dither_begin(1);
i_dither_end   = find(t_dither >= end_t); 
i_dither_end   = i_dither_end(1);
span_dither    = i_dither_begin : sample_T : i_dither_end;

% Plot for the dither case for variables a_1 to a_4 
plot(t_dither(span_dither, :)/3600, x_dither(span_dither, 5:8));
grid on
xlim([begin_t end_t]/3600)

% Compute maximum constraint values
constraint_max = constraint_no_dither - Umax;
constraint_max(constraint_max < 0) = 0;

% Create a new figure and apply standard figure settings
figure;
figure_standard

% Plot absolute value of maximum constraint values over time
plot(t_no_dither(1:100:720000)/3600, abs(constraint_max(1:100:720000)), 'b', 'LineWidth', 2); 
grid on;
xlabel('t [h]', 'interpreter', 'latex')
ylabel('$\max \{0,\, \sum_{i = 1}^4 b_i x_i - x_{\max}\}$', 'interpreter', 'latex')

% Create a subplot for zoomed-in views of specific time ranges
lower_plot2 = axes('Position',[.4 .62 .5 .25]); 
box on
plot(t_no_dither(700000:10:720000)/3600, abs(constraint_max(700000:10:720000)), 'b', 'LineWidth', 2);

lower_plot1 = axes('Position',[.45 .22 .45 .25]);
box on
plot(t_no_dither(300000:100:400000)/3600, abs(constraint_max(300000:100:400000)), 'b', 'LineWidth', 2);
