% Initializing variables

% Set the number of inputs for each agent
n       = [1 1 1 1 1 1 1 1]; 
rng(13); % Set the seed for random number generation to ensure repeatability

n_sum   = sum(n);                   % Total number of variables
na      = 4;                        % Number of a variables
nb      = 4;                        % Number of b variables
kappa   = 1;                        % Variable for calculation
epsilon_e = 0.1 * ones(1, n_sum);   % Vector for epsilon values
epsilon_f = 0.01 * ones(1, n_sum);  % Vector for epsilon values
epsilon = 10;                       % Single value for epsilon
amp     = 10;                       % Amplitude for signal

% Define boundaries for a variables
amin    = 0.5 * amp;
amax    = 1 * amp;
amid    = (amin + amax)/2;
a_min   = amin * ones(na, 1);
a_max   = amax * ones(na, 1);

% Define the cost functions
J       = {};
J{1}    = @(u) - (-3.9e-7 * u(1, :).^4 + 2.1e-4 * u(1, :).^3 - 0.043 * u(1, :).^2 + 3.7 * u(1, :) + 12);
J{2}    = @(u) - (-1.3e-7 * u(2, :).^4 + 1e-4 * u(2, :).^3 - 0.028 * u(2, :).^2 + 3.1 * u(2, :) - 17);
J{3}    = @(u) - (-1.2e-7 * u(3, :).^4 + 1e-4 * u(3, :).^3 - 0.028 * u(3, :).^2 + 2.5 * u(3, :) -16);
J{4}    = @(u) - (-4e-7 * u(4, :).^4 + 1.8e-4 * u(4, :).^3 - 0.036 * u(4, :).^2 + 3.5 * u(4, :) + 10);

% Define the pseudogradient F
F       = @(zeta, a) [zeta; [(zeta(1) * a(1) - zeta(2) * a(2))/2 * [1; -1]; ...
    (zeta(3) * a(3) - zeta(4) * a(4)) * [1; -1]] .* zeta * 10 -  ...
    1./(a - a_min)/log(100) + 1./(a_max - a)/log(100)];

% Time related variables
time            = 35000; % Total time
time_interval   = [0, time]; % Time interval
j_lim           = 10000; % Maximum number of jumps

% Rectangular boundary, slight acceleration, "close initial point"
Umax        = 200;      % Upper limit for sum of variables
kmin        = 0.01;     % Lower limit for k variable
kmax        = 10000;    % Upper limit for k variable

% Constraint function g
g           = @(u) sum(u(1 : n_sum/2) .* [1; 2; 3; 4]) - Umax; 

% Set other parameters
m           = 1;
k_limit     = [kmin kmax];
k_dot       = 1000 * ones(m, 1);

% Gradient of function g
nabla_g     = @(u) [[1; 2; 3; 4]; zeros(n_sum/2, 1)];

% Initial values
u0          = [10 10 10 10 amid amid amid amid]'; 
z0          = [10 10 10 10 amid amid amid amid]'; 
w0          = zeros(m, 1);
lambda0     = 0.1 * ones(m, 1);
zeta0       = zeros(sum(n) / 2, 1);
k0          = kmin * ones(m, 1);
s0          = zeros(m ,1);
t0          = 0;

% Combine all initial states
States_init = [u0; z0; w0; lambda0; k0; s0; zeta0; t0]; 

% Define the indices
i = zeros(13, 1);
i(2) = i(1) + sum(n);
i(3) = i(2) + sum(n);
i(4) = i(3) + m;
i(5) = i(4) + m;
i(6) = i(5) + m;
i(7) = i(6) + m;
i(8) = i(7) + sum(n) / 2;
i(9) = i(8) + 1;

% Global variables for inputs and internal parameters
global param_input param_internal last_time;
param_input    = {};
param_internal = {};
last_time      = 0;

% Define the input parameters
param_input{1}  = n;
param_input{2}  = J;
param_input{3}  = m;
param_input{4}  = g;
param_input{5}  = nabla_g;
param_input{6}  = k_limit;
param_input{7}  = k_dot;
param_input{8}  = i;
param_input{9}  = epsilon;
param_input{10} = kappa;
param_input{11} = F;
param_input{12} = epsilon_e;
param_input{13} = epsilon_f;
param_input{14} = a_min;
param_input{15} = a_max;

% Simulation horizon
TSPAN = [0 0.01 time];
JSPAN = [0 j_lim];

% Rules for jumps
rule = 1;

% Set options for the ode solver
options = odeset('RelTol', 1e-7, 'MaxStep', .1);

% Simulate
[t,j,x] = hybrid_gnep_solver(@f_gnep, @g_gnep, @C_gnep, @D_gnep,...
    States_init, TSPAN, JSPAN, rule, options, 'ode45');

beep % Sound alert when simulation ends

% Plot solution
figure; figure_standard % position
clf
subplot(2,1,1), plotHarc(t/3600,j,x(:, 1:4));
legend('$x_1$', '$x_2$', '$x_3$', '$x_4$')  
grid on
ylabel('$x_i$', 'interpreter', 'latex')
xlabel('t [h]', 'interpreter', 'latex')
subplot(2,1,2), plotHarc(t/3600,j,x(:, 5:8));
grid on
xlabel('t [h]', 'interpreter', 'latex')
ylabel('$a_i$', 'interpreter', 'latex')
legend('$a_1$', '$a_2$', '$a_3$', '$a_4$')

figure(3);
hold on

% Define J_f as a vector function
J_f = @(u) [J{1}(u); J{2}(u);J{3}(u);J{4}(u)];
u = x(:, i(1) + 1 : i(2))';
a = x(:, i(1) + sum(n)/2 + 1 : i(2))';
mi  = sin(2 * pi * kappa * t' + [pi/4  -3*pi/4 3*pi/4 -pi/4 ]');
J_eval = J_f(u(1 : n_sum/2, :) + a .* mi);

% Plot the sum of extracted oil
plot(t, sum(J_eval))
