function [StatesDot] = dynamics_fullinfo_gnep(t, States, param)

    % Unpacking parameters from param argument 
    n           = param{1};     % Vector that contains the sum of u sizes
    F           = param{2};     % Vector of cost functions
    m           = param{3};     % Number of convex constraints
    g           = param{4};     % Vector of functions of convex constraints g(x)<= 0
    nabla_g     = param{5};     % Gradient of g
    k_limit     = param{6};     % Min and max values of the dual coefficients
    k_dot       = param{7};     % Rate of change of k
    i           = param{8};     % Indices used in iterations

    % Extracting state variables from the States vector
    u       = States(i(1) + 1 : i(2));  % Decision variables
    z       = States(i(2) + 1 : i(3));  % 
    w       = States(i(3) + 1 : i(4));  % 
    lambda  = States(i(4) + 1 : i(5));  % Lagrange multipliers associated with inequality constraints
    k       = States(i(5) + 1 : i(6));  % Dual coefficients
    
    StatesDot                  = zeros(size(States));                                   % Initialize StatesDot as zero vector
    StatesDot(i(1) + 1 : i(2)) = -u + z - F(u) - nabla_g(u)'* lambda;                   % Compute the derivative of decision variables
    StatesDot(i(2) + 1 : i(3)) = -z + u;                                                % Compute the derivative of Lagrange multipliers associated with equality constraints   
    StatesDot(i(3) + 1 : i(4)) = -w + lambda;                                           % Compute the derivative of slack variables
    StatesDot(i(4) + 1 : i(5)) = k .* (lambda .* (g(u) - lambda + w));                  % Compute the derivative of Lagrange multipliers associated with inequality constraints
    StatesDot(i(5) + 1 : i(6)) = (g(u) >= 0 & k >= k_limit(1) & k<=k_limit(2)) * k_dot; % Compute the derivative of dual coefficients

end

