function StatesDot = f_gnep(States, t)
    
    % Define global variable param
    global param;
     
    % Unpack parameters from param variable 
    n       = param{1};   % Vector that contains the sum of u sizes
    F       = param{2};   % Vector of cost functions
    m       = param{3};   % Number of convex constraints
    g       = param{4};   % Vector of functions of convex constraints g(x)<= 0
    nabla_g = param{5};   % Gradient of g
    k_dot   = param{7};   % Rate of change of k
    i       = param{8};   % Indices used in iterations
    
    % Extract state variables from the States vector
    u      = States(i(1) + 1 : i(2));  % Decision variables
    z      = States(i(2) + 1 : i(3));  % Z variable
    w      = States(i(3) + 1 : i(4));  % W variable
    lambda = States(i(4) + 1 : i(5));  % Lagrange multipliers associated with inequality constraints
    k      = States(i(5) + 1 : i(6));  % Dual coefficients
    s      = States(i(6) + 1 : i(7));  % State variables for time-dependent part of the system
           
    % Initialize StatesDot as zero vector
    StatesDot = zeros(size(States));

    StatesDot(i(1) + 1 : i(2)) = -u + z - F(u) - nabla_g(u)' * lambda;  % Compute the derivative of decision variables
    StatesDot(i(2) + 1 : i(3)) = -z + u;                                % Compute the derivative of z variable
    StatesDot(i(3) + 1 : i(4)) = -w + lambda;                           % Compute the derivative of w variable
    StatesDot(i(4) + 1 : i(5)) = k .* (lambda .* (g(u) - lambda + w));  % Compute the derivative of Lagrange multipliers associated with inequality constraints
    StatesDot(i(5) + 1 : i(6)) = sin(pi/2 * s) .* k_dot;                % Compute the derivative of dual coefficients, modulated by the sine of pi/2 * s
    
end
