function StatePlus = g_gnep(States)
    % Define global variable param
    global param;
    
    % Unpack parameters from param variable 
    g       = param{4};   % Vector of functions of convex constraints g(x)<= 0
    k_limit = param{6};   % Minimum and maximum values of the dual coefficients
    i       = param{8};   % Indices used in iterations
    epsilon = param{9};   % Parameter used for creating hysteresis
    
    % Extract state variables from the States vector
    u = States(i(1) + 1 : i(2));  % Decision variables
    k = States(i(5) + 1 : i(6));  % Dual coefficients
    s = States(i(6) + 1 : i(7));  % State variables for time-dependent part of the system
    
    % Copy current States vector to StatePlus
    StatePlus = States;
    
    % Evaluate function g at current u
    g_eval  = g(u);
    
    % Determine indices where state variables s need to be flipped (1 to 0 or 0 to 1)
    change_index = (g_eval >= 2 * epsilon & s == 0) | ...
        (g_eval <= epsilon & s == 1);
    
    % Flip state variables s at the indices determined above
    s(change_index) = 1 - s(change_index);
    
    % Determine indices where state variables s need to be set to 2 (this occurs when k has reached its upper limit)
    change_index = (k >= k_limit(2) & (s == 1 | s == 0));
    
    % Set state variables s to 2 at the indices determined above
    s(change_index) = 2;

    % Update StatePlus with new s
    StatePlus(i(6) + 1 : i(7)) = s;
end
