function [t, u, z, w, lambda, k, q] = one_forward_step_gnep_solver(States_init, param, time_interval)
    %% About the function
    % This function implements the one forwards step algorithm for monotone
    % games with constraints. In this version of the algorithm, a low
    % pass filter is applied on the gradient estimation.
    % All inputs, appart for the param, are selfexplanitory.
    % param is a cell variable that contains the following data:
    %     param{1} = n;         %vector that contains the length of sizes of "u" state
    %     param{2} = kappa;     %vector that contains the perturbation frequencies (one frequency per state)
    %     param{3} = A;         %vector that contains all perturbation amplitudes(one per agent)
    %     param{4} = epsilon;   %timescale seperation parameters between the oscilator dynamics and rest of the system (one per agent)
    %     param{5} = J;         %vector of cost functions. All should have the same argument "u" even in the MATLAB code.
    %     param{6} = epsilon_f  %vector of timescale seperation parameters between the gradient filter and the rest of the system
    %     param{7} = m          %number of convex constraints
    %     param{8} = g          %vector that constaints all convex constraints
    %     param{9} = epsilon_w  %vector of timescale seperation parameters between the lagrangian dynamics and other dynamics. Is a product of two different constants
    %     param{10}= nabla_g    %gradient of constraints
    %% TODO Working example
    % 
      
    n = param{1}; %vector that contains the sum of u sizes
    m = param{3}; %number of convex constraints
    q = param{8};
    
    % Summation of n. The algorithm only needs to know the complete size of
    % the whole system.
    param{1} = sum(n);

    % Indices used in every interations
    i = zeros(7, 1);
    i(2) = i(1) + sum(n);
    i(3) = i(2) + sum(n);
    i(4) = i(3) + m;
    i(5) = i(4) + m;
    i(6) = i(5) + m;
    i(7) = i(6) + q;
    param{size(param, 2) + 1} = i;
    
    options = odeset('RelTol',1e-7);
    
 [t,States] = ode45(@(t,y)dynamics_fullinfo_gnep(t, y, param), time_interval, States_init, options);
  u         = States(:, i(1) + 1 : i(2));
  z         = States(:, i(2) + 1 : i(3));
  w         = States(:, i(3) + 1 : i(4));
  lambda    = States(:, i(4) + 1 : i(5));
  k         = States(:, i(5) + 1 : i(6));
  q         = States(:, i(6) + 1 : i(7));
end

