% Find indices where jumps occur
jump_index = find(diff(j_adp)) + 1;

% Set the sampling rate and the ending time for the plots
sample_T = 20;
end_t    = 50;

% Find the indices corresponding to the ending time in different time vectors
i_1e5 = find(t_1e5>=end_t);
i_1e5 = i_1e5(1);

i_100 = find(t_100>=end_t);
i_100 = i_100(1);

i_1 = find(t_1>=end_t);
i_1 = i_1(1);

i_adp = find(t_adp>=end_t);
i_adp = i_adp(1);

%%

% Create new figure for plotting
figure 
clf

% Plot the first set of data
subplot(2,1,1)  % First subplot for u1 variable
hold on
% Plot different trajectories for the first decision variable u1
plot(t_1e5(1:sample_T:i_1e5), x_1e5(1:sample_T:i_1e5, 1), 'b', 'LineWidth', 2)
plot(t_100(1:sample_T:i_100), x_100(1:sample_T:i_100, 1), 'y', 'LineWidth', 2)
plot(t_1(1:sample_T:i_1), x_1(1:sample_T:i_1, 1), 'm', 'LineWidth', 2)
plot(t_adp(1:sample_T:i_adp), x_adp(1:sample_T:i_adp, 1), 'r', 'LineWidth', 2)
plot(t_adp(jump_index), x_adp(jump_index, 1), 'r*', 'MarkerSize', 10, 'LineWidth', 1)
plot([t_1(1) t_1(i_1)], [4 4], 'k--', 'LineWidth', 1)  % Draw horizontal line at u1=4

% Add grid, labels, legend, and set limits
grid on
ylabel('$u_1$','interpreter','latex')
xlabel('$t$','interpreter','latex')
legend({'Flow in (10), $k = 10^5$', 'Flow in (10), $k = 10^2$', 'Flow in (10), $k = 1$', 'Flow in (13)', 'Jumps in (14)', 'GNE'}, 'NumColumns', 2)
xlim([0, end_t])

% Plot the second set of data
subplot(2,1,2)  % Second subplot for u2 variable
hold on
% Plot different trajectories for the second decision variable u2
plot(t_1e5(1:sample_T:i_1e5), x_1e5(1:sample_T:i_1e5, 2), 'b', 'LineWidth', 2)
plot(t_100(1:sample_T:i_100), x_100(1:sample_T:i_100, 2), 'y', 'LineWidth', 2)
plot(t_1(1:sample_T:i_1), x_1(1:sample_T:i_1, 2), 'm', 'LineWidth', 2)
plot(t_adp(1:sample_T:i_adp), x_adp(1:sample_T:i_adp, 2), 'r', 'LineWidth', 2)
plot(t_adp(jump_index), x_adp(jump_index, 2), 'r*', 'MarkerSize', 10, 'LineWidth', 1)
plot([t_1(1) t_1(i_1)], [3 3], 'k--', 'LineWidth', 1)  % Draw horizontal line at u2=3

% Add grid, labels, and set limits
grid on
ylabel('$u_2$','interpreter','latex')
xlabel('$t$','interpreter','latex')
figure_standard  % Standardize the figure according to custom function
xlim([0, end_t])

%%
% Plot multiple trajectories at once
figure
hold on

xlim([2.5 5])
ylim([1.8 4.2])

% Create shaded regions using patches
v = [-0.5 -1.5; -0.5 -5; 6 -5; 6 5];
f = [1 2 3 4];
h = patch('Faces',f,'Vertices',v,'FaceColor','green', 'EdgeColor', 'none', 'FaceAlpha', 0.1);
h.Annotation.LegendInformation.IconDisplayStyle = 'off';

v = [4 -5; 4 6; 6 6; 6 -5];
f = [1 2 3 4];
h = patch('Faces',f,'Vertices',v,'FaceColor','red', 'EdgeColor', 'none', 'FaceAlpha', 0.1);
h.Annotation.LegendInformation.IconDisplayStyle = 'off';

% Plot different trajectories in the u1-u2 plane
plot(x_1e5(1:sample_T:i_1e5, 1), x_1e5(1:sample_T:i_1e5, 2), 'b', 'LineWidth', 2)
plot(x_100(1:sample_T:i_1, 1), x_100(1:sample_T:i_1, 2), 'y', 'LineWidth', 2)
plot(x_1(1:sample_T:i_1, 1), x_1(1:sample_T:i_1, 2), 'm', 'LineWidth', 2)
plot(x_adp(1:sample_T:i_adp, 1), x_adp(1:sample_T:i_adp, 2), 'r', 'LineWidth', 2)
plot(x_adp(jump_index,1),x_adp(jump_index,2), 'r*', 'MarkerSize', 10, 'LineWidth', 1)

% Mark the point (4, 3) with a yellow dot
plot(4, 3, 'ko', 'MarkerFaceColor', 'yellow', 'MarkerSize', 5)

% Add grid and labels
grid on
ylabel('$u_2$','interpreter','latex')
xlabel('$u_1$','interpreter','latex')
figure_standard  % Standardize the figure according to custom function

lower_plot = axes('Position',[.15 .55 .4 .4]); 
box on
%%
hold on
grid on
plot(x_1e5(1:sample_T:i_1e5, 1), x_1e5(1:sample_T:i_1e5, 2), 'b', 'LineWidth', 2)
plot(x_100(1:sample_T:i_1, 1), x_100(1:sample_T:i_1, 2), 'y', 'LineWidth', 2)
plot(x_1(1:sample_T:i_1, 1), x_1(1:sample_T:i_1, 2), 'm', 'LineWidth', 2)
plot(x_adp(1:sample_T:i_adp, 1), x_adp(1:sample_T:i_adp, 2), 'r', 'LineWidth', 2)
xlim([-0.5, 6])
ylim([-5, 6])

% Create shaded regions using patches
v = [-0.5 -1.5; -0.5 -5; 6 -5; 6 5];
f = [1 2 3 4];
h = patch('Faces',f,'Vertices',v,'FaceColor','green', 'EdgeColor', 'none', 'FaceAlpha', 0.1);
h.Annotation.LegendInformation.IconDisplayStyle = 'off';

v = [4 -5; 4 6; 6 6; 6 -5];
f = [1 2 3 4];
h = patch('Faces',f,'Vertices',v,'FaceColor','red', 'EdgeColor', 'none', 'FaceAlpha', 0.1);
h.Annotation.LegendInformation.IconDisplayStyle = 'off';


plot(4, 3, 'ko', 'MarkerFaceColor', 'yellow', 'MarkerSize', 5)
% set(gca,'xticklabel',[])
% set(gca,'yticklabel',[])
ylabel('$u_2$','interpreter','latex')
xlabel('$u_1$','interpreter','latex')
figure_standard




%% Plot the gains
figure
plot(t_adp(1:sample_T:i_adp), x_adp(1:sample_T:i_adp, i(5) + 1 : i(6)), 'LineWidth', 2);
grid on
ylabel('$k_1,\, k_2$','interpreter','latex')
xlabel('$t$','interpreter','latex')
legend('Gain $k_1$', 'Gain $k_2$')
figure_standard  % Standardize the figure according to custom function
