% Initialization of parameters

% System order for each agent
n           = [1 1]; 

% Define the function g
g           = @(u) [u(2) - u(1) + 1; 4 - u(1)]; 

% Define the gradient of function g
nabla_g     = @(u) [-1 1; -1 0];

% Constant q
q           = 2;

% m dimension
m           = 2;

% Define the pseudogradient F
F           = @(u) [u(2) + 3; -u(1) + 2];

% Define the simulation time and interval
time            = 200;
time_interval   = [0, time];

% Define the limit of k and rate of change of k
k_limit         = [1 1000];
k_dot           = 10;

% Define the parameter epsilon
epsilon         = 0.2;

% Define the maximal number of jumps
j_lim           = 100;

% Initialize the state variables
States_init = [[10 0 0 0]'; [0.1 0.1 0.1 0.1]'; [k_limit(1) k_limit(1)]'; zeros(q, 1)]; 

% Define a parameter set
param    = {};
param{1} = n;         
param{2} = F;     
param{3} = m;         
param{4} = g;   
param{5} = nabla_g;  
param{6} = k_limit;
param{7} = k_dot;

% Define indices for different state variables
i = zeros(7, 1);
i(2) = i(1) + sum(n);
i(3) = i(2) + sum(n);
i(4) = i(3) + m;
i(5) = i(4) + m;
i(6) = i(5) + m;
i(7) = i(6) + q;

% Define a global parameter set
global param
param    = {};
param{1} = sum(n);         
param{2} = F;     
param{3} = m;         
param{4} = g;   
param{5} = nabla_g;  
param{6} = k_limit;
param{7} = k_dot;
param{8} = i;
param{9} = epsilon;

% Define simulation horizon
TSPAN = [0 time];
JSPAN = [0 j_lim];

% Define rule for jumps and flows
% rule = 1 -> priority for jumps
% rule = 2 -> priority for flows
rule = 1;

% Define options for ODE solver
options = odeset('RelTol',1e-7,'MaxStep',.01);

% Run the simulation using Hybrid Equation (HyEQ) solver
[t,j,x] = HyEQsolver( @f_gnep, @g_gnep, @C_gnep, @D_gnep, States_init, TSPAN, JSPAN, rule, options, 'ode45');

% Plot the phase plane
figure(3) 
hold on
plot(x(:,1), x(:,2));
xlabel('u_1')
ylabel('u_2')
title('Phase plane')
grid on
