function [inside] = C(States)
    %----------------------------------------------------------------------
    % Description: Flow set
    % Determines if the state is within the flow set.
    % The function returns 0 if the state is outside of the flow set C, and 1 if it's inside C.
    %----------------------------------------------------------------------

    % Access global parameters
    global param;

    % Extract relevant indices and time constants from the global parameters
    i       = param{2};    % Parameter indices
    T       = param{3};    % Vector of sampling periods
    T_w     = param{16};   % Vector of sampling periods for the unicycle reference controler

    % Extract relevant states
    tau     = States(i(2) + 1 : i(3));     % timer state
    tau_w   = States(i(10) + 1 : i(11));   % timer state

    % Check if each state is less than or equal to its corresponding period
    S   = tau <= T';
    Sw  = tau_w <= T_w';

    % If the sum of S and Sw is greater than or equal to 1, the state is inside the flow set
    if  sum(S + Sw) >= 1 
        inside = 1;
    else
        inside = 0;
    end
end
