function StatesDot = f(States, t)
    %----------------------------------------------------------------------
    % Description: Derivative of states
    % This function calculates the derivatives of the system's states
    %----------------------------------------------------------------------

    % Access global parameters
    global param;

    N = param{8};  % Number of agents
    i = param{2};  % Parameter indices

    % Extract relevant states
    theta   = States(i(7) + 1 : i(8));  % Theta (orientation) states
    w       = States(i(8) + 1 : i(9));  % Angular velocity states
    v       = States(i(9) + 1 : i(10)); % Velocity states

    % Initialize StatesDot as a zero vector with the same size as States
    StatesDot = zeros(size(States));
    
    % Assign values to elements of StatesDot based on the state variables
    StatesDot(i(2) + 1 : i(3))      = ones(N, 1);                  % Timer flow for N states
    StatesDot(i(6) + 1 : 2 : i(7))  = v .* cos(theta);             % Velocity component in x-direction
    StatesDot(i(6) + 2 : 2 : i(7))  = v .* sin(theta);             % Velocity component in y-direction
    StatesDot(i(7) + 1 : i(8))      = w;                           % Angular velocities
    StatesDot(i(10) + 1 : i(11))    = ones(N, 1);                  % Timer flow for  for N states
    StatesDot(i(11) + 1 : i(12))    = 1;                           % Time flow

end
