m_i         = 2;                    % Define the number of control input states per vehicle
N           = 4;                    % Define the number of vehicles
m           = N * m_i;              % Total number of control input states
n_i         = 3;                    % Define the number of non-control input states per vehicle
n           = N * n_i;              % Total number of non-control input states
rng(13);                            % Seed the random number generator for reproducibility

% Constructing the cost function
a = 0.08;                           % Define cost coefficient

% Define the matrix Q for the cost function
Q_proto = -eye(m_i) * a;
Q = repmat(Q_proto, N, N);
Q(eye(m) == 1) = (1 + (N - 1) * a/2); 

% Define the R matrix for the cost function
R_s = [-4, -8; -12, -3; 1, 7; 16, 8]';  % Define the raw values of R
R = R_s(:);                             % Flatten R_s to a column vector

% Indexing control input states and non-control input states
index     = 1:m;
index_all = index;
index_j   = reshape(index_all, m_i, [])';
index_minus_j = zeros(m_i * (N - 1), N);
for j = 1:N
    temp = index_j';
    temp(:, j) = [];
    index_minus_j(:, j) = temp(:);
end
index_minus_j = index_minus_j';
index_reg = reshape(1 : m_i * N, m_i, [])';

% Constructing the cost function
J  =  @(x)[[1; 1] * (x(index_j(1, :))' * Q(index_reg(1, :), :) * x(index_all) + (R(index_reg(1, :)) - 2 * x(index_j(1, :)))' *  R(index_reg(1, :)) + a/2 * x(index_minus_j(1, :))' * x(index_minus_j(1, :))); ...
          [1; 1] * (x(index_j(2, :))' * Q(index_reg(2, :), :) * x(index_all) + (R(index_reg(2, :)) - 2 * x(index_j(2, :)))' *  R(index_reg(2, :)) + a/2 * x(index_minus_j(2, :))' * x(index_minus_j(2, :))); ...
          [1; 1] * (x(index_j(3, :))' * Q(index_reg(3, :), :) * x(index_all) + (R(index_reg(3, :)) - 2 * x(index_j(3, :)))' *  R(index_reg(3, :)) + a/2 * x(index_minus_j(3, :))' * x(index_minus_j(3, :))); ...
          [1; 1] * (x(index_j(4, :))' * Q(index_reg(4, :), :) * x(index_all) + (R(index_reg(4, :)) - 2 * x(index_j(4, :)))' *  R(index_reg(4, :)) + a/2 * x(index_minus_j(4, :))' * x(index_minus_j(4, :)))]; %...

Qsol = Q;
Qsol(eye(m) == 1) = Qsol(eye(m) == 1) *2;
NE = 2 * inv(Qsol) * R;
NE = reshape(NE, 2, []);

%% Parameters for the equilibirum seeking
Ri  = cell(m);
for i = 1:m
    Ri{i} = [cos(phi(i)) -sin(phi(i)); sin(phi(i)) cos(phi(i))];
end
Rcal  = diag_matrices(Ri);

time            = 200;               % Define the simulation time
time_interval   = [0, time];         % Define the simulation time interval
j_lim           = 500000;            % Define the maximum number of jumps
gamma           = 0.1;               % Define the learning rate for pseudogradient descent
sigma           = 0.1;               % Define the learning rate
T               = [0.01, 0.015, 0.02, 0.01]; % Define the sampling time for the agent controller
A               = 0.1;               % Define the amplitude of the perturbation signals
beta            = 0.003;             % Define the time seperation constant
tau0            = [0 0.002 0.004 0.006]'; % Define the initial values of tau

u0 = zeros(m, 1);                   % Initialize control input
x0 = zeros(n, 1);                   % Initialize states
States_init = [u0; tau0; ...
               zeros(m * 1, 1); repmat([1; 0], m, 1); zeros(m, 1); ...
               u0 ; zeros(N, 1); zeros(N, 1); zeros(N, 1); tau0 * 5; 0]; 

c2      = T/5;
c3      = 1/3;
c1      = 1/2/c3;
gamma_w = 1;
T_w     = T/5;    

% Define indices for each part of the state vector
i = zeros(3, 1);
i(2) = i(1) + m;        %u
i(3) = i(2) + N;        %tau
i(4) = i(3) + m;        %xi
i(5) = i(4) + 2 * m;    %mi
i(6) = i(5) + m;        %k
i(7) = i(6) + m;        %x and y
i(8) = i(7) + N;        %theta
i(9) = i(8) + N;        %w
i(10) = i(9) + N;       %v
i(11) = i(10) + N;      %tau_w
i(12) = i(11) + 1;      %t

global param

param    = {};           % Define a cell array to store parameters
param{1} = m;         
param{2} = i;     
param{3} = T;         
param{4} = gamma;   
param{5} = J;
param{6} = Rcal;
param{7} = m_i;
param{8} = N;
param{9} = A;
param{10} = beta;
param{11} = phi;

param{12} = c1;
param{13} = c2;
param{14} = c3;
param{15} = gamma_w;
param{16} = T_w;

% Simulation horizon
TSPAN = [0 time];
JSPAN = [0 j_lim];

% Rule for jumps
% Rule = 1 -> priority for jumps
% Rule = 2 -> priority for flows
rule = 1;

options = odeset('MaxStep',.01);

% Simulate
tic
[t,j,x] = HyEQsolver( @f, @g, @C, @D,...
    States_init, TSPAN, JSPAN, rule, options, 'ode45');
toc




































last_index = size(x, 1);
%% u states, time evolution
figure
plot(t(1:100:end), x(1:100:end, i(1) + 1 : i(2)))
xlabel('t')
ylabel('x')
title('Time evolution')
grid on


% Get the last index of x
last_index = size(x, 1);

% Create a new figure to plot time evolution of u states
figure
plot(t(1:100:end), x(1:100:end, i(1) + 1 : i(2))) % Plot u states against time (every 100th data point)
xlabel('t')                                      % Label the x-axis as 't'
ylabel('x')                                      % Label the y-axis as 'x'
title('Time evolution')                          % Set the plot title as 'Time evolution'
grid on                                          % Turn on the grid

% Create a new figure to plot phase plane of x states with the last state
plot_sample = 100;                               % Set the plot sampling rate
figure                                           % Create a new figure
figure_standard                                  % Apply standard figure settings
hold on                                          % Hold the figure for multiple plots
% Plot the x states in the phase plane
plot(x(1:plot_sample:last_index, i(6) + 1), x(1:plot_sample:last_index, i(6) + 2), 'b', 'LineWidth', 2)
plot(x(1:plot_sample:last_index, i(6) + 3), x(1:plot_sample:last_index, i(6) + 4), 'r', 'LineWidth', 2)
plot(x(1:plot_sample:last_index, i(6) + 5), x(1:plot_sample:last_index, i(6) + 6), 'Color', [0.9290 0.6940 0.1250], 'LineWidth', 2)
plot(x(1:plot_sample:last_index, i(6) + 7), x(1:plot_sample:last_index, i(6) + 8), 'm', 'LineWidth', 2)

% Plot the source signal locations and equilibrium points
plot(R_s(1, :), R_s(2, :), 'o', 'HandleVisibility','off', 'Color', 'b', 'MarkerSize', 10)
plot(NE(1, :), NE(2, :), 'x', 'HandleVisibility','off', 'Color', 'b', 'MarkerSize', 10)
xlabel('$x_i$')                                 % Label the x-axis as 'x_i'
ylabel('$y_i$')                                 % Label the y-axis as 'y_i'
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4') % Add a legend to the plot
grid on                                         % Turn on the grid

% Create a new figure to plot time evolution of x states
figure                                           % Create a new figure
figure_standard                                  % Apply standard figure settings
hold on                                          % Hold the figure for multiple plots

% Plot the x states against time
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(6) + 1), 'b', 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(6) + 3), 'r', 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(6) + 5), 'Color', [0.9290 0.6940 0.1250], 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(6) + 7), 'm', 'LineWidth', 2)

% Plot Nash equilibrium points
plot([t(1) , t(last_index)], [NE(1, 1) NE(1, 1)], '--b', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 1) NE(2, 1)], '--b', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 2) NE(1, 2)], '--r', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 2) NE(2, 2)], '--r', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 3) NE(1, 3)], 'LineStyle', '--', 'Color', [0.9290 0.6940 0.1250], 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 3) NE(2, 3)], 'LineStyle', '--', 'Color', [0.9290 0.6940 0.1250], 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 4) NE(1, 4)], '--m', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 4) NE(2, 4)], '--m', 'HandleVisibility','off')
xlabel('$t$')                                   % Label the x-axis as 't'
ylabel('$x$')                                   % Label the y-axis as 'x'
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4') % Add a legend to the plot
grid on                                         % Turn on the grid


