function [v_proj] = Proj_polyhedron2D(v, x, A, B)
%%
% Calculates the projection of the vector v onto the tangent cone of x in
% a 2d convex polyhedron Ax <= B
    

    answers = A * x >= B;
    if (~answers)
        v_proj = v;
    else 
        wrong = sum(answers);
        a = A(answers, :);
        b = B(answers);
        if wrong == 1
            v_proj = (x + v) + (b - a * (x + v)) *  a' /(a * a') - x;
        elseif wrong == 2
            a1 = A(1, :); 
            b1 = b(1, 1);
            a2 = A(2, :);
            b2 = b(2, 1);
            
            proj1 = (x + v) + (b1 - a1 * (x + v)) *  a1' /(a1 * a1');
            proj2 = (x + v) + (b2 - a2 * (x + v)) *  a2' /(a2 * a2');
            
            con1 = a2 * proj1 <= b2;
            con2 = a1 * proj2 <= b1;
            
            if(~con1 && ~con2)
                v_proj = [0; 0];
            elseif(con1)
            	v_proj = proj1 - x;
            else
            	v_proj = proj2 - x; 
            end
        else
            error('Error. \n Two many planes intersect.')
        end
    end
    
    
    
end
