function [x, y] = give_elipse(C, R)
%%
% Gives the x, y coordinates of a 2D elipse given by the general formula
% $$ (x - C)^\top diag(1/R.^2)\ (x - C) = 1, $$
% where C is the center of the elipsoid 
% For example:
% (x-1)^2/a^2 + (x - 2)^2/b^2 = 1 would have C = [1; 2] and R = [a, b]).
% 
    
%     phi = 0:pi/10:2 * pi;
%     x = C(1) + R(1) * cos(phi);
%     y = C(2) + R(2) * sin(phi);
    
    
    phi = (0:pi/100:2 * pi)';
    X = [cos(phi) sin(phi)];
    Y = X * chol(R)^(-1);
    
    
    x = C(1) + Y(:, 1);
    y = C(2) + Y(:, 2);
    
    
    
end
