function [x] = proj_elipse(x, C, R)
%%
% Calculates the projection of the point x onto the N-dimensional elipsoid
% given in the most general form by:\\
% $$ (x - C)^\top R\ (x - C) = 1, $$
% where C is the center of the elipsoid and $R = (AA^\top)^{-1}$
% For example:
% (x-1)^2/a^2 + (x - 2)^2/b^2 = 1 would have C = [1; 2] and A = diag(a, b).
% 
    
    alpha = (x - C)' * R * (x - C);
    % if x is inside the circle, return the point it self, if not, process
    % it further
    if (alpha > 1)
        k = 1/sqrt(alpha);
        x = C + (x - C) * k;
    end
    
end
